<?php

defined('_VALID_CALL') or die('Direct Access is not allowed.');

if($page->page_name == 'product')
{
    global $page, $p_info, $price, $system_status, $currency;

    /**
     *  Artikel
     */
    $product_ld = [
        "@context" => "https://schema.org/",
        "@type" => "Product",
        "name" => $p_info->data["products_name"],
        "description" => trim( $p_info->data["products_short_description"]),
        "sku" => $p_info->data["products_model"]
    ];

    //  Artikel > Beschreibung
    $desc = trim(strip_tags($p_info->data["products_short_description"]));
    if(!empty($desc))
        $desc = trim($p_info->data["products_short_description"]);
    else
        $desc = trim(strip_tags($p_info->data["products_description"]));

    if(!empty($desc))
        $product_ld["description"] =$desc;

    //  Artikel > gtin/ean, mpn
    // fbo in meinen tests hat google immer nur die spezifische gtin erkannt also zb gtin8
    // wir suchen erst nach der spezifischen und fallen dann auf einfache gtin zurück
    $gtin_lengths = [8,12,13,14];
    $gtin = trim($p_info->data["products_ean"]);
    if(!empty($gtin))
    {
        if(in_array(strlen($gtin), $gtin_lengths))
            $product_ld["gtin".strlen($gtin)] = $gtin;
        else
            $product_ld["gtin"] = $gtin;
    }

    $mpn = trim($p_info->data["products_mpn"]);
    if(!empty($mpn))
    {
        $product_ld["mpn"] = $mpn;
    }

    //  Artikel > Marke
    if (!empty($p_info->data["manufacturers_id"]))
    {
        $name = $db->CacheGetOne("SELECT manufacturers_name FROM " . TABLE_MANUFACTURERS . " WHERE manufacturers_id=?", [$p_info->data["manufacturers_id"]]);
        $product_ld["brand"] = [
            "@type" => "Organization",
            "name" => $name
        ];
    }

    //  Artikel > Bilder
    $xt_images[] = $p_info->data["products_image"];
    if(!empty($p_info->data["more_images"]))
    {
        foreach($p_info->data["more_images"] as $v)
            $xt_images[] = $v['file'];
    }
    $imgParams = [
        'type' => 'm_info',
        'img' => "",
        'path_only' => true,
        'return' => true
    ];
    $images = [];
    foreach($xt_images as $v)
    {
        if ($v != 'product:noimage.gif' && !empty($v))
        {
            $imgParams["img"] = $v;
            $imgParams["type"] = "m_info";
            $img_info = image::getImgUrlOrTag($imgParams);
            $imgParams["type"] = "m_thumb";
            $img_thumb = image::getImgUrlOrTag($imgParams);
            $images[] = [
                "@type" => "ImageObject",
                "name" => $p_info->data["products_name"],
                "url" => $img_info,
                "thumbnail" => [
                    //"@type" => "ImageObject",
                    //"name" => $p_info->data["products_name"],
                    "url" => $img_thumb
                ]
            ];
        }
    }
    if (count($images))
        $product_ld["image"] = $images;

    /**
     *  Artikel > Rating
     */
    if($p_info->data["products_rating_count"] > 0)
        $product_ld["aggregateRating"] = [
            "@type" => "AggregateRating",
            "ratingValue" => $p_info->data["products_average_rating"],
            "reviewCount" => $p_info->data["products_rating_count"],
            "bestRating" =>  5,
            "worstRating" => 0
          ];


    /**
     *   Artikel > Angebot
     */
    $product_ld["offers"] = [
        "@type" => "Offer",
        "url" => $p_info->data["products_link"],
        "priceCurrency" => $currency->default_currency,
        "price" => round($p_info->data["products_price"]["plain"], 2)
      ];

    //  Artikel > Angebot > gültig bis
    if(isset($p_info->data["products_price"]["date_expired"]))
    {
        try{
            $priceValidUntil = DateTime::createFromFormat("Y-m-d h:i:s", $p_info->data["products_price"]["date_expired"]);
            if(is_object($priceValidUntil))
            {
                $priceValidUntil = $priceValidUntil->format("Y-m-d h:i:s");
                $product_ld["offers"]["priceValidUntil"] = $priceValidUntil;
            }
        }
        catch(Exception $e){}
    }

    //  Artikel > Angebot > Verkäufer
    $seller = trim(constant("_STORE_SHOPOWNER_COMPANY"));
    if(!empty($seller))
    {
        $product_ld["offers"]["seller"] = [
            "@type" => "Organization",
            "name" => $seller
        ];
    }

    //  Artikel > Angebot > Verfügbarkeit, Zustand
    $condition = !empty($p_info->data["products_condition"]) ?
        "https://schema.org/".$p_info->data["products_condition"] : constant("_STORE_DEFAULT_PRODUCT_CONDITION");
    if(!empty($condition))
    {
        $product_ld["offers"]["itemCondition"] = $condition;
    }
    $availability = !empty($p_info->data["mapping_schema_org"]) ?
        "https://schema.org/".$p_info->data["mapping_schema_org"] : "";
    if(!empty($availability))
    {
        $product_ld["offers"]["availability"] = $availability;
    }


    echo PHP_EOL.'<script type="application/ld+json">'. PHP_EOL.json_encode($product_ld, JSON_PRETTY_PRINT|JSON_UNESCAPED_SLASHES). PHP_EOL.'</script>'. PHP_EOL;
}
